using System;
using Microsoft.SPOT;

namespace SwimTrainer
{
	/// <summary>
	/// This class manage a Stopwatch
	/// </summary>
	public class Chrono
	{
		public event ChronoDelegate IntervalElapsed; 

		readonly TimeSpan _Interval = new TimeSpan(0, 0, 1);
		TimeSpan _PauseDelta;
		ExtendedTimer _Timer;
		//System.Threading.Timer _Timer;
		DateTime _Reference;
		TimeSpan _Elapsed;
		ChronoState _State;
		ChronoEventArgs _EventArgs;
		TimeSpanFormatter _TimeSpanFormatter;
		object _Sync = new object();

		public Chrono()
		{
			_Timer = new ExtendedTimer(new System.Threading.TimerCallback(Timer_Elapsed), null, System.Threading.Timeout.Infinite, System.Threading.Timeout.Infinite);
			//_Timer = new System.Threading.Timer(new System.Threading.TimerCallback(Timer_Elapsed), null, System.Threading.Timeout.Infinite, System.Threading.Timeout.Infinite);

			_Elapsed = TimeSpan.Zero;
			_State = ChronoState.Stopped;

			_EventArgs = new ChronoEventArgs();
			_TimeSpanFormatter = new TimeSpanFormatter();
		}

		public void Start()
		{
			lock(_Sync)
			{
				Debug.Print("Chrono Start");
				if(_State == ChronoState.Started || _State == ChronoState.Running)
					return;

				_Reference = DateTime.Now;

				if(_State == ChronoState.Paused)
				{
					if(_Interval > _PauseDelta)
						_Timer.Change(_Interval - _PauseDelta, _Interval);
					else
						_Timer.Change(_PauseDelta, _Interval);
				}
				else
				{
					_Elapsed = TimeSpan.Zero;
					_Timer.Change(_Interval, _Interval);
				}

				_State = ChronoState.Started;
				OnIntervalElapsed();
			}
		}

		public void Pause()
		{
			lock(_Sync)
			{
				DateTime Temp = DateTime.Now;
				_Timer.Change(System.Threading.Timeout.Infinite, System.Threading.Timeout.Infinite);// stop

				Debug.Print("Chrono Pause");
				if(_State == ChronoState.Paused || _State == ChronoState.Stopped)
					return;

				_PauseDelta = Temp - _Reference;
				_Elapsed += _PauseDelta;
				_State = ChronoState.Paused;

				OnIntervalElapsed();
			}
		}

		public void Stop()
		{
			lock(_Sync)
			{
				DateTime Temp = DateTime.Now;
				_Timer.Change(System.Threading.Timeout.Infinite, System.Threading.Timeout.Infinite);// stop

				Debug.Print("Chrono Stop");
				if(_State == ChronoState.Stopped)
					return;
				if(_State == ChronoState.Paused)
				{
					_PauseDelta = TimeSpan.Zero;
					_State = ChronoState.Stopped;
					return;
				}

				_Elapsed += Temp - _Reference;
				_PauseDelta = TimeSpan.Zero;
				_State = ChronoState.Stopped;

				OnIntervalElapsed();
			}
		}

		public ChronoState State
		{
			get {return _State;}
		}

		public TimeSpan Elapsed
		{
			get { return _Elapsed; }
		}

		void Timer_Elapsed(object state)
		{
			lock(_Sync)
			{
				_Elapsed += _Interval;
				_Reference = DateTime.Now;

				_State = ChronoState.Running;
				OnIntervalElapsed();
				Debug.Print("Thread = " + Dispatcher.CurrentDispatcher.CheckAccess().ToString());
			}
		}

		protected virtual void OnIntervalElapsed()
		{
			Debug.Print("OnIntervalElapsed");
			if(IntervalElapsed != null)
			{
				_EventArgs.Elapsed = _Elapsed;
				_EventArgs.State = _State;
				IntervalElapsed(this, _EventArgs);
				Debug.Print(Dispatcher.CurrentDispatcher.CheckAccess().ToString());
			}
		}

		public string GetFormattedTime(bool IncludeMilliseconds)
		{
			return _TimeSpanFormatter.GetFormattedTime(_Elapsed, IncludeMilliseconds);
		}
	}

	public enum ChronoState
	{
		Started,
		Running,
		Paused,
		Stopped,
	}
	public delegate void ChronoDelegate(object sender, ChronoEventArgs e);
	public class ChronoEventArgs : EventArgs
	{
		TimeSpan _Elapsed;
		ChronoState _State;

		public TimeSpan Elapsed
		{
			get { return _Elapsed; }
			set { _Elapsed = value; }
		}
		public ChronoState State
		{
			get { return _State; }
			set { _State = value; }
		}
	}

}
