using System;
using Microsoft.SPOT;
using Microsoft.SPOT.Input;
using Microsoft.SPOT.Hardware;
using Microsoft.SPOT.Presentation;
using iMXsBoard = Microsoft.SPOT.Hardware.FreescaleMXSDemo;

//
//  This class should be defined on a per-OEM basis based on their hardware behavior.
//

namespace SwimTrainer
{
	public sealed class GPIOButtonInputProvider
	{
		public readonly Dispatcher Dispatcher;

		private ButtonPad[] buttons;
		private ReportInputCallback callback;
		private InputProviderSite site;
		private PresentationSource source;

		InterruptPort _PlayPausePort;
		InterruptPort _StopPort;


		private delegate bool ReportInputCallback(InputReport inputReport);

		public GPIOButtonInputProvider(PresentationSource source)
		{
			this.source = source;
			site = InputManager.CurrentInputManager.RegisterInputProvider(this);
			callback = new ReportInputCallback(site.ReportInput);
			Dispatcher = Dispatcher.CurrentDispatcher;

			ButtonPad[] buttons = new ButtonPad[]
			{
				#region iMXS Board specific button mappings
				new ButtonPad(this, Button.Left  , iMXsBoard.Pins.GPIO_PORT_B_16),
				new ButtonPad(this, Button.Right , iMXsBoard.Pins.GPIO_PORT_B_12),
				new ButtonPad(this, Button.Up    , iMXsBoard.Pins.GPIO_PORT_B_11),
				new ButtonPad(this, Button.Select, iMXsBoard.Pins.GPIO_PORT_B_8),
				new ButtonPad(this, Button.Down  , iMXsBoard.Pins.GPIO_PORT_B_10),

				new ButtonPad(this, Button.Menu  , iMXsBoard.Pins.GPIO_PORT_B_13),

				//new ButtonPad(this, Button.Play  , iMXsBoard.Pins.GPIO_PORT_B_9),		// handled manually (see below)
				//new ButtonPad(this, Button.Stop  , iMXsBoard.Pins.GPIO_PORT_B_17),	// handled manually (see below)

				new ButtonPad(this, Button.Rewind  , iMXsBoard.Pins.GPIO_PORT_B_14),
				new ButtonPad(this, Button.FastForward  , iMXsBoard.Pins.GPIO_PORT_B_15),
				#endregion
			};

			_PlayPausePort = new InterruptPort(iMXsBoard.Pins.GPIO_PORT_B_9, false, Port.ResistorMode.PullUp, Port.InterruptMode.InterruptEdgeBoth);
			_StopPort = new InterruptPort(iMXsBoard.Pins.GPIO_PORT_B_17, false, Port.ResistorMode.PullUp, Port.InterruptMode.InterruptEdgeBoth);

			this.buttons = buttons;
		}

		public InterruptPort PlayPausePort
		{
			get { return _PlayPausePort; }
		}
		public InterruptPort StopPort
		{
			get { return _StopPort; }
		}


		internal class ButtonPad
		{
			private Button button;
			private InterruptPort port;
			private GPIOButtonInputProvider sink;

			public ButtonPad(GPIOButtonInputProvider sink, Button button, Cpu.Pin pin)
			{
				this.sink = sink;
				this.button = button;

				port = new InterruptPort(pin, false, Port.ResistorMode.PullUp, Port.InterruptMode.InterruptEdgeBoth);

				port.OnInterrupt += new GPIOInterruptEventHandler(this.Interrupt);
			}

			void Interrupt(Cpu.Pin port, bool state, TimeSpan time)
			{
				RawButtonActions action = state ? RawButtonActions.ButtonUp : RawButtonActions.ButtonDown;

				RawButtonInputReport report = new RawButtonInputReport(sink.source, time, button, action);

				// queue the button press to the input provider site.
				sink.Dispatcher.BeginInvoke(sink.callback, report);
			}
		}
	}
}


