using System;

using Microsoft.SPOT;
using Microsoft.SPOT.Input;
using Microsoft.SPOT.Presentation;
using Microsoft.SPOT.Presentation.Controls;
using Microsoft.SPOT.Presentation.Media;
using Microsoft.SPOT.Hardware;


namespace SwimTrainer
{
	public class Program : Microsoft.SPOT.Application
	{
		private Window _MainWindow;
		GPIOButtonInputProvider _InputProvider;
		Text _ClockText;
		Chrono _Chrono;
		SwimTimingCollection _SwimTimings;
		ChronoDelegate _ChronoDel;
		object[] _DelArgs;
		TimeSpanFormatter _TimeSpanFormatter;
		ListBox _CurrentListBox;

		static readonly Color _UnselectedItemColor = ColorUtility.ColorFromRGB(255, 168, 168);
		static readonly Color _UnselectedItemColorStart = ColorUtility.ColorFromRGB(255, 128, 128);
		static readonly Color _UnselectedItemColorEnd = ColorUtility.ColorFromRGB(40, 0, 0);
		static readonly Brush _UnselectedItemBrush = new LinearGradientBrush(_UnselectedItemColorStart, _UnselectedItemColorEnd, 0, 0, 0/*SystemMetrics.ScreenWidth*/, SystemMetrics.ScreenHeight);

		static readonly Color _SelectedItemColor = ColorUtility.ColorFromRGB(255, 255, 0);
		static readonly Color _SelectedItemColorStart = ColorUtility.ColorFromRGB(128, 64, 64);
		static readonly Color _SelectedItemColorEnd = ColorUtility.ColorFromRGB(20, 0, 0);
		static readonly Brush _SelectedItemBrush = new LinearGradientBrush(_SelectedItemColorStart, _SelectedItemColorEnd, 0, SystemMetrics.ScreenHeight / 2, 0/*SystemMetrics.ScreenWidth*/, SystemMetrics.ScreenHeight / 2);

		ListBox _HistoryListBox;
		ListBox _StatisticsListBox;

		private static ExtendedWeakReference _CriticalData;

		public static void Main()
		{
			Debug.Print("Main");
			Program myApplication = new Program();

			Window mainWindow = myApplication.CreateWindow();
		//	myApplication.InsertFakeData();

			Application.Current.Startup += new EventHandler(Current_Startup);

			// Avvia l'applicazione
			myApplication.Run(mainWindow);
		}

		/// <summary>
		/// Application Startup event
		/// </summary>
		static void Current_Startup(object sender, EventArgs e)
		{
			Debug.Print("Startup");
		}

		private void InsertFakeData()
		{
			Microsoft.SPOT.Math.Randomize();
			DateTime dt = new DateTime(0, 0, 0, 18, 30, 0);
			TimeSpan ts;

			ts = new TimeSpan(0, 0, 1, 20, 0);
			dt = dt.Add(ts);
			dt = dt.AddSeconds(Microsoft.SPOT.Math.Random(20));
			ts = ts.Add(new TimeSpan(0, 0, 0, Microsoft.SPOT.Math.Random(5), Microsoft.SPOT.Math.Random(99)));
			AddNewTime(new SwimTiming(dt, ts));

			ts = new TimeSpan(0, 0, 1, 20, 0);
			dt = dt.Add(ts);
			dt = dt.AddSeconds(Microsoft.SPOT.Math.Random(20));
			ts = ts.Add(new TimeSpan(0, 0, 0, Microsoft.SPOT.Math.Random(5), Microsoft.SPOT.Math.Random(99)));
			AddNewTime(new SwimTiming(dt, ts));

			ts = new TimeSpan(0, 0, 1, 20, 0);
			dt = dt.Add(ts);
			dt = dt.AddSeconds(Microsoft.SPOT.Math.Random(20));
			ts = ts.Add(new TimeSpan(0, 0, 0, Microsoft.SPOT.Math.Random(5), Microsoft.SPOT.Math.Random(99)));
			AddNewTime(new SwimTiming(dt, ts));

			ts = new TimeSpan(0, 0, 1, 20, 0);
			dt = dt.Add(ts);
			dt = dt.AddSeconds(Microsoft.SPOT.Math.Random(20));
			ts = ts.Add(new TimeSpan(0, 0, 0, Microsoft.SPOT.Math.Random(5), Microsoft.SPOT.Math.Random(99)));
			AddNewTime(new SwimTiming(dt, ts));

			ts = new TimeSpan(0, 0, 1, 20, 0);
			dt = dt.Add(ts);
			dt = dt.AddSeconds(Microsoft.SPOT.Math.Random(20));
			ts = ts.Add(new TimeSpan(0, 0, 0, Microsoft.SPOT.Math.Random(5), Microsoft.SPOT.Math.Random(99)));
			AddNewTime(new SwimTiming(dt, ts));

			ts = new TimeSpan(0, 0, 1, 20, 0);
			dt = dt.Add(ts);
			dt = dt.AddSeconds(Microsoft.SPOT.Math.Random(20));
			ts = ts.Add(new TimeSpan(0, 0, 0, Microsoft.SPOT.Math.Random(5), Microsoft.SPOT.Math.Random(99)));
			AddNewTime(new SwimTiming(dt, ts));

			ts = new TimeSpan(0, 0, 1, 20, 0);
			dt = dt.Add(ts);
			dt = dt.AddSeconds(Microsoft.SPOT.Math.Random(20));
			ts = ts.Add(new TimeSpan(0, 0, 0, Microsoft.SPOT.Math.Random(5), Microsoft.SPOT.Math.Random(99)));
			AddNewTime(new SwimTiming(dt, ts));

			ts = new TimeSpan(0, 0, 1, 20, 0);
			dt = dt.Add(ts);
			dt = dt.AddSeconds(Microsoft.SPOT.Math.Random(20));
			ts = ts.Add(new TimeSpan(0, 0, 0, Microsoft.SPOT.Math.Random(5), Microsoft.SPOT.Math.Random(99)));
			AddNewTime(new SwimTiming(dt, ts));

			ts = new TimeSpan(0, 0, 1, 20, 0);
			dt = dt.Add(ts);
			dt = dt.AddSeconds(Microsoft.SPOT.Math.Random(20));
			ts = ts.Add(new TimeSpan(0, 0, 0, Microsoft.SPOT.Math.Random(5), Microsoft.SPOT.Math.Random(99)));
			AddNewTime(new SwimTiming(dt, ts));

			ts = new TimeSpan(0, 0, 1, 20, 0);
			dt = dt.Add(ts);
			dt = dt.AddSeconds(Microsoft.SPOT.Math.Random(20));
			ts = ts.Add(new TimeSpan(0, 0, 0, Microsoft.SPOT.Math.Random(5), Microsoft.SPOT.Math.Random(99)));
			AddNewTime(new SwimTiming(dt, ts));
		}

		/// <summary>
		/// Constructor. Data initialization
		/// </summary>
		public Program()
		{
			_InputProvider = new GPIOButtonInputProvider(null);
			_InputProvider.PlayPausePort.OnInterrupt += new Microsoft.SPOT.Hardware.GPIOInterruptEventHandler(PlayPause_OnInterrupt);
			_InputProvider.StopPort.OnInterrupt += new Microsoft.SPOT.Hardware.GPIOInterruptEventHandler(Stop_OnInterrupt);

			_ChronoDel = new ChronoDelegate(Chrono_IntervalElapsed_Dispatched);
			_DelArgs = new object[2];
			_TimeSpanFormatter = new TimeSpanFormatter();

			// trying to restore data from flash memory
			_CriticalData = ExtendedWeakReference.RecoverOrCreate(typeof(SwimTimingCollection), 0, ExtendedWeakReference.c_SurvivePowerdown);
			_CriticalData.Priority = (int)ExtendedWeakReference.PriorityLevel.Important;
			SwimTimingCollection LoadedData = _CriticalData.Target as SwimTimingCollection;
			_SwimTimings = new SwimTimingCollection();
			if(LoadedData != null)
			{
				// if I find data in the flash memory, I copy them into main memory
				foreach(SwimTiming st in LoadedData)
				{
					SwimTiming stnew = new SwimTiming(st);
					AddNewTime(stnew);
				}
			}

			_CriticalData.Target = _SwimTimings;
		}

		public Window CreateWindow()
		{
			// Window
			_MainWindow = new Window();
			_MainWindow.Height = SystemMetrics.ScreenHeight;
			_MainWindow.Width = SystemMetrics.ScreenWidth;
			_MainWindow.Background = _UnselectedItemBrush;

			// StackPanel that contains all the controls
			StackPanel spv = new StackPanel(Orientation.Vertical);
			_MainWindow.Child = spv;

			// Upper element: Listbox 2/3 high that will show the time history
			_HistoryListBox = new ListBox();
			_HistoryListBox.Height = ((int)(SystemMetrics.ScreenHeight * 2 / 3) / 15) * 15 + 1;	// 15 + 4

			//_HistoryListBox.VerticalAlignment = VerticalAlignment.Stretch;
			_HistoryListBox.Width = SystemMetrics.ScreenWidth;
			_HistoryListBox.HorizontalAlignment = HorizontalAlignment.Center;
			_HistoryListBox.Background = _UnselectedItemBrush;
			_HistoryListBox.ScrollingStyle = ScrollingStyle.LineByLine;
			_HistoryListBox.SelectionChanged += new SelectionChangedEventHandler(ListBox_SelectionChanged);
			spv.Children.Add(_HistoryListBox);

			// Lower panel (used to create the black border)
			Border BottomBorder = new Border();
			BottomBorder.SetBorderThickness(0, 1, 0, 0);
			BottomBorder.Background = _UnselectedItemBrush;
			spv.Children.Add(BottomBorder);

			// Lower Stackpanel (1/3 high) oriented from left to right
			StackPanel sph = new StackPanel(Orientation.Horizontal);
			sph.Height = SystemMetrics.ScreenHeight - _HistoryListBox.Height;
			BottomBorder.Child = sph;

			Border LeftBorder = new Border();
			LeftBorder.SetBorderThickness(1);
			LeftBorder.Background = _UnselectedItemBrush;
			sph.Children.Add(LeftBorder);

			// Left element inside the lower StackPanel
			_StatisticsListBox = new ListBox();
			_StatisticsListBox.Width = SystemMetrics.ScreenWidth / 4;
			_StatisticsListBox.Height = SystemMetrics.ScreenHeight - _HistoryListBox.Height;
			_StatisticsListBox.VerticalAlignment = VerticalAlignment.Stretch;
			_StatisticsListBox.Background = _UnselectedItemBrush;
			_StatisticsListBox.ScrollingStyle = ScrollingStyle.LineByLine;
//			_StatisticsListBox.SetMargin(10, 0, 0, 0);
			_StatisticsListBox.SelectionChanged += new SelectionChangedEventHandler(ListBox_SelectionChanged);

			// Listbox items
			int HItems = _StatisticsListBox.Height / 4 - 4;
			_StatisticsListBox.Items.Add(CreateLowerItem(_UnselectedItemBrush, "Last", _StatisticsListBox.Width, HItems, HorizontalAlignment.Left));
			_StatisticsListBox.Items.Add(CreateLowerItem(_UnselectedItemBrush, "Average", _StatisticsListBox.Width, HItems, HorizontalAlignment.Left));
			_StatisticsListBox.Items.Add(CreateLowerItem(_UnselectedItemBrush, "Lower", _StatisticsListBox.Width, HItems, HorizontalAlignment.Left));
			_StatisticsListBox.Items.Add(CreateLowerItem(_UnselectedItemBrush, "Higher", _StatisticsListBox.Width, HItems, HorizontalAlignment.Left));
			ScrollViewer sv = _StatisticsListBox.Items[0].Parent.Parent as ScrollViewer;
			sv.Background = _UnselectedItemBrush;

			LeftBorder.Child = _StatisticsListBox;

			// Lower-right panel that contains text
			Border TextBorder = new Border();
			TextBorder.SetBorderThickness(0, 1, 2, 0);
			TextBorder.Background = _UnselectedItemBrush;
			TextBorder.Width = SystemMetrics.ScreenWidth - GetListboxWidth(_StatisticsListBox);
			sph.Children.Add(TextBorder);

			// text with the stopwatch or the other statistics items
			_ClockText = CreateText("", HorizontalAlignment.Center, VerticalAlignment.Center);
			TextBorder.Child = _ClockText;

			// First item selected
			_StatisticsListBox.SelectedIndex = 0;
			_CurrentListBox = _StatisticsListBox;

			// Make the main window visible
			_MainWindow.Visibility = Visibility.Visible;
			// Buttons Event Handlers
			_MainWindow.AddHandler(Buttons.ButtonUpEvent, new ButtonEventHandler(OnButtonUp), false);
			Buttons.Focus(_MainWindow);

			// Chrono initializations
			_Chrono = new Chrono();
			_Chrono.IntervalElapsed += new ChronoDelegate(Chrono_IntervalElapsed);

			return _MainWindow;
		}

		void ListBox_SelectionChanged(object sender, SelectionChangedEventArgs args)
		{
			ListBox lb = sender as ListBox;
			if(args.PreviousSelectedIndex != -1)
			    lb.Items[args.PreviousSelectedIndex].Background = _UnselectedItemBrush;
			lb.Items[args.SelectedIndex].Background = _SelectedItemBrush;
			lb.Invalidate();

			if(lb.Equals(_StatisticsListBox) && (args.SelectedIndex >= 0))
			{
				switch(args.SelectedIndex)
				{
					case 0:	// Last
						{
							SwimTiming st = _SwimTimings.GetLast();
							if(st != null)
								_ClockText.TextContent = st.ToString(true);
							else
								_ClockText.TextContent = string.Empty;
						}
						break;
					case 1:	// Average
						_ClockText.TextContent = _TimeSpanFormatter.GetFormattedTime(_SwimTimings.GetAverage(), true);
						break;
					case 2:	// Min
						_ClockText.TextContent = _TimeSpanFormatter.GetFormattedTime(_SwimTimings.GetMin(), true); ;
						break;
					case 3:	// Max
						_ClockText.TextContent = _TimeSpanFormatter.GetFormattedTime(_SwimTimings.GetMax(), true);
						break;
				}
			}
		}

		#region UI Utility functions
		ListBoxItem CreateLowerItem(Brush Background, string Content, int Width, int Height, HorizontalAlignment HAlign)
		{
			ListBoxItem lbi = new ListBoxItem();
			lbi.Background = Background;
			lbi.HorizontalAlignment = HorizontalAlignment.Left;
			lbi.SetMargin(5, 2, 5, 2);
			lbi.Child = CreateText(Content, HAlign, VerticalAlignment.Bottom);
			if(Width > 0)
				lbi.Width = Width;
			if(Height > 0)
				lbi.Height = Height;
			return lbi;
		}

		ListBoxItem CreateUpperItem(Brush Background, string Content1, string Content2, int Width, int Height, HorizontalAlignment HAlign)
		{
			ListBoxItem lbi = new ListBoxItem();
			lbi.Background = Background;
			lbi.HorizontalAlignment = HorizontalAlignment.Left;
			lbi.SetMargin(10, 2, 10, 2);

			StackPanel sp = new StackPanel(Orientation.Horizontal);
			Text t1 = CreateText(Content1, HorizontalAlignment.Right, VerticalAlignment.Bottom);
			Border b = new Border();
			b.Background = Background;
			b.SetBorderThickness(0);
			b.HorizontalAlignment = HorizontalAlignment.Right;
			b.Width = 30;
			b.Child = t1;
			sp.Children.Add(b);
			Image img = new Image(Resources.GetBitmap(Resources.BitmapResources.clock));
			sp.Children.Add(img);


			Text t2 = CreateText(Content2, HAlign, VerticalAlignment.Bottom);
			t2.Width = SystemMetrics.ScreenWidth - b.Width;
			sp.Children.Add(t2);

			lbi.Child = sp;
			if(Width > 0)
				lbi.Width = Width;
			if(Height > 0)
				lbi.Height = Height;
			return lbi;
		}


		Text CreateText(string Content, HorizontalAlignment HAlign, VerticalAlignment VAlign)
		{
			Text txt = new Text();
			txt.Font = Resources.GetFont(Resources.FontResources.NinaB);
			txt.TextContent = Content;
			txt.HorizontalAlignment = HAlign;
			txt.VerticalAlignment = VAlign;
			txt.ForeColor = Color.White;
			txt.SetMargin(6, 0, 6, 0);
			return txt;
		}

		int GetListboxWidth(ListBox lb)
		{
			int w, h;
			lb.Measure(Int32.MaxValue, Int32.MaxValue);
			lb.GetDesiredSize(out w, out h);
			return w;
		}

		int GetListboxHeight(ListBox lb)
		{
			int w, h;
			lb.Measure(Int32.MaxValue, Int32.MaxValue);
			lb.GetDesiredSize(out w, out h);
			return h;
		}


		void AddNewTime(SwimTiming st)
		{
			_SwimTimings.Add(st);
			_HistoryListBox.Items.Insert(0, CreateUpperItem(_UnselectedItemBrush, st.Number.ToString(), st.ToString(), _HistoryListBox.Width, 0, HorizontalAlignment.Left));
			//_HistoryListBox.SelectedIndex = index;

			ScrollViewer sv = _HistoryListBox.Items[0].Parent.Parent as ScrollViewer;

			sv.Background = _UnselectedItemBrush;
			sv.LineHeight = _HistoryListBox.Items[_HistoryListBox.Items.Count-1].ActualHeight + 4;	// 4  il margine
			sv.ScrollingStyle = ScrollingStyle.LineByLine;
//			sv.Invalidate();
//			sv.LineDown();
		}

		//void sv_ScrollChanged(object sender, ScrollChangedEventArgs args)
		//{
		//    ScrollViewer sv = sender as ScrollViewer;
		//    //sv.LineDown();
		//}

		#endregion

		// Chrono class event
		void Chrono_IntervalElapsed(object sender, ChronoEventArgs e)
		{
			_DelArgs[0] = sender;
			_DelArgs[1] = e;
			this.Dispatcher.BeginInvoke(_ChronoDel, _DelArgs);
		}

		// Method executed in the main thread context
		void Chrono_IntervalElapsed_Dispatched(object sender, ChronoEventArgs e)
		{
			_ClockText.TextContent = _Chrono.GetFormattedTime(e.State == ChronoState.Paused || e.State == ChronoState.Stopped);
			_ClockText.UpdateLayout();
			if(e.State == ChronoState.Stopped)
				AddNewTime(new SwimTiming(DateTime.Now, e.Elapsed));
		}


		private void OnButtonUp(object sender, ButtonEventArgs e)
		{
			Debug.Print(e.Button.ToString());
			if(e.Button == Button.Down)		// _StatisticsListBox
			{
				if(_StatisticsListBox.SelectedIndex == _StatisticsListBox.Items.Count - 1)
					return;
				else
					_StatisticsListBox.SelectedIndex++;
			}

			if(e.Button == Button.Up)		// _StatisticsListBox
			{
				if(_StatisticsListBox.SelectedIndex == 0)
					return;
				else
					_StatisticsListBox.SelectedIndex--;
			}


			if(e.Button == Button.FastForward)		// _HistoryListBox
			{
				if(_HistoryListBox.Items.Count == 0)
					return;
				ScrollViewer sv = _HistoryListBox.Items[0].Parent.Parent as ScrollViewer;
				sv.LineDown();
				sv.Invalidate();
			}

			if(e.Button == Button.Rewind)	// _HistoryListBox
			{
				if(_HistoryListBox.Items.Count == 0)
					return;
				ScrollViewer sv = _HistoryListBox.Items[0].Parent.Parent as ScrollViewer;
				sv.LineUp();
				sv.Invalidate();
			}

			if(e.Button == Button.Select)
				Microsoft.SPOT.Hardware.Utility.HibernateSystem();

//			if(e.Button == Button.Left)
//				;

			if(e.Button == Button.Right)		// upload via serial port
			{
				try
				{
					Upload();
				}
				catch(ConstraintException err)
				{
					_ClockText.TextContent = "ConstraintException " + err.ToString();
				}
				catch(Exception err)
				{
					_ClockText.TextContent = "Exception " + err.ToString();
				}
			}

			if(e.Button == Button.Menu)
			{
				_SwimTimings = new SwimTimingCollection();
				_HistoryListBox.Items.Clear();
				_StatisticsListBox.SelectedIndex = 0;
				//_ClockText.TextContent = string.Empty;
			}

		}

		private void Upload()
		{
			ExecutionConstraint.Install(2000, 1);
			SerialPort.Configuration config = new SerialPort.Configuration(SerialPort.Serial.COM1, SerialPort.BaudRate.Baud19200, false);
			SerialPort port = new SerialPort(config);
			byte[] Blob = Reflection.Serialize(_SwimTimings, typeof(SwimTimingCollection));
			port.Write(Blob, 0, Blob.Length);

			port.Dispose();
		}

		void PlayPause_OnInterrupt(Microsoft.SPOT.Hardware.Cpu.Pin port, bool state, TimeSpan time)
		{
			if(!state)	// we are only interesed to the released switch event
				return;
			if(_Chrono.State == ChronoState.Running || _Chrono.State == ChronoState.Started)
			{
				_Chrono.Pause();
				return;
			}
			_Chrono.Start();
		}

		void Stop_OnInterrupt(Microsoft.SPOT.Hardware.Cpu.Pin port, bool state, TimeSpan time)
		{
			_Chrono.Stop();
		}

	}


}
