using System;
using Microsoft.SPOT;
using System.Collections;

namespace SwimTrainer
{
	[Serializable]
	public class SwimTiming : IComparable
	{
		int _Number;
		DateTime _TimeStart;
		TimeSpan _Elapsed;

		[NonSerialized]
		SwimTimingCollection _List;

		public SwimTiming(DateTime TimeStart, TimeSpan Elapsed)
		{
			_TimeStart = TimeStart;
			_Elapsed = Elapsed;
		}

		public SwimTiming(SwimTiming st)
		{
			_Number = st.Number;
			_TimeStart = st.TimeStart;
			_Elapsed = st.Elapsed;
		}

		public int Number
		{
			get { return _Number; }
			set { _Number = value; }
		}

		public DateTime TimeStart
		{
			get { return _TimeStart; }
		}

		public TimeSpan Elapsed
		{
			get { return _Elapsed; }
		}

		internal SwimTimingCollection List
		{
			get { return _List; }
			set { _List = value; }
		}

		public int CompareTo(object obj)
		{
			SwimTiming st = obj as SwimTiming;
			if(st == null)
				throw new ArgumentException();
			return DateTime.Compare(this.TimeStart, st.TimeStart);
		}
		public override string ToString()
		{
			return
				_TimeStart.ToString("HH:mm") + "    " +
				_List.Formatter.GetFormattedTime(_Elapsed, true) + "   (" +
				GetDifferenceFromPrevious() + ")";
		}

		private string GetDifferenceFromPrevious()
		{
			TimeSpan Diff;
			SwimTiming Prev = List.Previous(this);
			if(Prev == null)
				Diff = TimeSpan.Zero;
			else
				Diff = Prev.Elapsed;

			if(Elapsed >= Diff)
				return "+" + _List.Formatter.GetFormattedTime(Elapsed - Diff, true);

			return "-" + _List.Formatter.GetFormattedTime(Diff - Elapsed, true);
		}

		public string ToString(bool ElapsedOnly)
		{
			if(!ElapsedOnly)
				return ToString();
			return _List.Formatter.GetFormattedTime(_Elapsed, true);
		}

	}

	[Serializable]
	public class SwimTimingCollection : ArrayList
	{
		int Counter = 0;

		[NonSerialized]
		TimeSpanFormatter _TimeSpanFormatter;

		public SwimTimingCollection()
		{
			_TimeSpanFormatter = new TimeSpanFormatter();
		}

		internal TimeSpanFormatter Formatter
		{
			get { return _TimeSpanFormatter; }
		}

		public override int Add(object value)
		{
			SwimTiming st = value as SwimTiming;
			st.Number = ++Counter;
			st.List = this;
			return base.Add(value);
		}

		internal SwimTiming GetLast()
		{
			if(Count == 0)
				return null;
			return this[Count - 1] as SwimTiming;
		}

		internal TimeSpan GetAverage()
		{
			long TotalTicks = 0;
			foreach(object obj in this)
			{
				SwimTiming st = obj as SwimTiming;
				TotalTicks += st.Elapsed.Ticks;
			}
			if(TotalTicks == 0)
				return TimeSpan.Zero;
			return new TimeSpan(TotalTicks / Count);
		}

		internal TimeSpan GetMin()
		{
			long min = long.MaxValue;
			foreach(object obj in this)
			{
				SwimTiming st = obj as SwimTiming;
				min = (st.Elapsed.Ticks < min) ? st.Elapsed.Ticks : min;
			}
			if(min == long.MaxValue)
				return TimeSpan.Zero;
			return new TimeSpan(min);
		}

		internal TimeSpan GetMax()
		{
			long max = 0;
			foreach(object obj in this)
			{
				SwimTiming st = obj as SwimTiming;
				max = (st.Elapsed.Ticks > max) ? st.Elapsed.Ticks : max;
			}
			return new TimeSpan(max);
		}

		internal SwimTiming Previous(SwimTiming swim)
		{
			SwimTiming Temp = null;
			foreach(object obj in this)
			{
				SwimTiming st = obj as SwimTiming;
				if(st.Equals(swim))
					return Temp;
				Temp = st;
			}
			return null;
		}
	}

}
